#!/usr/bin/env python3

from vismach import *
import hal
import math
import sys

# Machine zero as measured from center surface of the rotary c table
machine_zero_x = -1000
machine_zero_y =  1000
machine_zero_z =  2000

for setting in sys.argv[1:]: exec(setting)

c = hal.component("xyzab-srt-gui")
# axis_x
c.newpin("axis_x", hal.HAL_FLOAT, hal.HAL_IN)
# axis_y
c.newpin("axis_y", hal.HAL_FLOAT, hal.HAL_IN)
# head vertical slide
c.newpin("axis_z", hal.HAL_FLOAT, hal.HAL_IN)
# rotary_a
c.newpin("rotary_a", hal.HAL_FLOAT, hal.HAL_IN)
# rotary_b
c.newpin("rotary_b", hal.HAL_FLOAT, hal.HAL_IN)
# tool offsets
c.newpin("tool_length", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("tool_diameter", hal.HAL_FLOAT, hal.HAL_IN)
# geometric offsets in the spindle-rotary-assembly
c.newpin("pivot_y", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("pivot_z", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("offset_x", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("offset_z", hal.HAL_FLOAT, hal.HAL_IN)
c.ready()

# give endpoint Z values and radii
# resulting cylinder is on the Z axis
class HalToolCylinder(CylinderZ):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 6 # default if hal pin not set
        if (c.tool_diameter > 0): r = c.tool_diameter/2
        return -c.tool_length, r, 0, r

# used to create an indicator for the rotational axis a
# this will be rotated 45° later
class HalRotAxisA(CylinderZ):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 2
        # calculate length from the pivot-point to the tool-axis
        # note: sin(45°) = 1/sqrt(2)
        length = sqrt(2) * c.pivot_y
        return 0, r, length, r

# used to create the spindle housing as set by the variable 'pivot_z'
class HalSpindleHousingZ(CylinderZ):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 140
        length = c.pivot_z + 200
        # start the spindle housing at hight 100 above the spindle nose
        return length, r, 100, r

# used to create the spindle rotary as set by the variable 'pivot_y'
class HalSpindleHousingY(CylinderY):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 140
        length = -c.pivot_y
        return length, r, 0, r

# used to create an indicator for the variable 'pivot_z'
class HalPivotZ(CylinderZ):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 2
        length = c.pivot_z
        # start the spindle housing at hight 100 above the spindle nose
        return length, 1, 0, r

# used to create an indicator for the variable 'pivot_y'
class HalPivotY(CylinderY):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 2
        length = -c.pivot_y
        return length, r, 0, 1

# used to create an indicator for the variable 'offset_x'
class HalOffsetX(CylinderX):
    def __init__(self, comp, *args):
        CylinderX.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 2
        length = -c.offset_x
        return length, r, 0, 1

# used to create an indicator for the variable 'offset_z'
class HalOffsetZ(CylinderZ):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 2
        length = -c.offset_z
        return length, r, 0, 1

# used to create a thin tool-offset indicator
class HalToolOffset(CylinderZ):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 2
        return -c.tool_length, 1, 0, r

class CoordSystem(Collection):
    # creates a visual object for a coordinate system
    def __init__(self, comp, r=4, l=500):
        # set arrow length
        al = l/10
        self.parts = [Color([1,0,0],[CylinderX(0,r,l,r)]),
                      Color([1,0,0],[CylinderX(l,2*r,l+al,1)]),
                      Color([0,1,0],[CylinderY(0,r,l,r)]),
                      Color([0,1,0],[CylinderY(l,2*r,l+al,1)]),
                      Color([0,0,1],[CylinderZ(0,r,l,r)]),
                      Color([0,0,1],[CylinderZ(l,2*r,l+al,1)]),
                     ]

class Point(Collection):
    # creates a visual object for a point
    def __init__(self, comp, r=1, l=20, color=[1,1,1]):
        self.parts = [Color(color,[CylinderX(-l,r,l,r)]),
                      Color(color,[CylinderY(-l,r,l,r)]),
                      Color(color,[CylinderZ(-l,r,l,r)])
                     ]

#indicators
# create vismach coordinates to show the origin during construction of the model
vismach_coords = CoordSystem(c,2,5000)

# create tcp coordinates
tcp_coords = CoordSystem(c,6,100)
# create tool coordinates that represent the oriented tool coordinate system
# ()these will rotate with the a- and b-axis rotation)
tool_coords = CoordSystem(c,2,100)
# rotate to match the current tool orientation
tool_coords = HalRotate([tool_coords],c,"rotary_a",1,0,1,1)
tool_coords = HalRotate([tool_coords],c,"rotary_b",1,0,1,0)
# add a visual indicator for the position of the tool-rotation point
tool_coords = Collection([
              tool_coords,
              Point(c,1,30,[1,0.8,0,1])
              ])
coords = Collection([
              tcp_coords,
              tool_coords
              ])
# move the above indicators to by the designated (x,y,z)-home offset
coords = Translate([coords], machine_zero_x, machine_zero_y,  machine_zero_z)


# create a visual indicator for the position of the control point
ctrl_pt = Point(c)
# move the control point indicator to the control-position
#ctrl_pt = Translate([ctrl_pt], machine_zero_x, machine_zero_y,  machine_zero_z)
ctrl_pt = HalTranslate([ctrl_pt],c,"tool_length",0,0,-1)


#/indicators

#tool-side
tooltip = Capture()
tool_cylinder = Collection([
                HalToolCylinder(c),
                # this indicates the spindle nose when no tool-offset is active
                CylinderZ(-0.1, 0, 0.1, 10),
                ])
# create an indicator for the tool-offset when spindle-body is hidden
ind_tool_offset = Color([1,0,1,0],[HalToolOffset(c)])
tool = Collection([
       HalTranslate([tooltip], c, "tool_length", 0,0,-1),
       tool_cylinder,
       ind_tool_offset
       ])
tool = Color([1,0,1,0], [tool] )
# create an indicator for the variable pivot_z
ind_pivot_z = Color([0,0,0.75,1],[HalPivotZ(CylinderZ)])
# create the spindle housing that contains the spindle

spindle_housing = Collection([
                  tool,
                  Color([0.7,0.7,0,1],[CylinderZ(0,20,50,50)]),
                  ind_pivot_z,
                  ctrl_pt
                  ])
# move the spindle and it's vertical housing by the values set for the pivot lengths
# so the vismach origin is in the pivot point
spindle_housing = HalTranslate([spindle_housing],c,"pivot_y",0,-1,0)
spindle_housing = HalTranslate([spindle_housing],c,"pivot_z",0,0,-1)
# spindle housing horizontal length as set by 'pivot_y'
spindle_housing_horizontal = Color([0.7,0.7,0,1],[HalSpindleHousingY(c)])
# create an indicator for the variable pivot_z
ind_pivot_y = Color([0,0.75,0,1],[HalPivotY(c)])
spindle_housing = Collection([
                  spindle_housing,
                  ind_pivot_y
                  ])
# create a visual for the position of the spindle A,B pivot-point
ind_pivot_point = Collection([
                  Color([1,1,1,1],[CylinderX(-200,1,200,1)]),
                  Color([1,1,0.3,1],[CylinderY(0,2,3000,2)]),
                  Color([1,1,1,1],[CylinderZ(-200,1,200,1)]),
                  # arrow indicating the up position
                  Color([1,1,1,1],[CylinderZ(200,5,210,1)]),
                  # sphere for the actual pivot point
                  Color([1,1,0.3,1],[Sphere(0,0,0,5)])
                  ])
# create HAL-link for a-axis rotational joint"
spindle_housing = HalRotate([spindle_housing],c,"rotary_a",1,1,0,0)
# move the joint using the offset values as set by the user in the gui panel
spindle_housing = HalTranslate([spindle_housing],c,"offset_x",-1,0,0)
spindle_housing = HalTranslate([spindle_housing],c,"offset_z",0,0,-1)


# create an indicator for x-offset
ind_offset_x =  Color([1,0,0,1],[HalOffsetX(c)])
# create an indicator for z-offset
ind_offset_z =  Color([0,1,1,1],[HalOffsetZ(c)])
ind_offset_z = HalTranslate([ind_offset_z],c,"offset_x", -1,0,0)

# create an indicator for rotational axis A
ind_axis_a = Color([1,1,0.3,1],[HalRotAxisA(c)])
ind_axis_a = Rotate([ind_axis_a],90,0,1,0)

# move the a axis indicator using the offset values as set by the user in the gui panel
ind_axis_a =  HalTranslate([ind_axis_a],c,"offset_x",-1,0,0)
ind_axis_a =  HalTranslate([ind_axis_a],c,"offset_z",0,0,-1)

spindle_rotary = Collection([
                 ind_offset_x,
                 ind_offset_z,
                 ind_pivot_point,
                 ind_axis_a,
                 ])
# join the two parts to a spindle assembly
spindle_assembly = Collection([
                   spindle_housing,
                   spindle_rotary,
                   ])
# create HAL-link for b-axis rotational joint"
spindle_assembly = HalRotate([spindle_assembly],c,"rotary_b",1,0,1,0)
# add a block for the y-axis to the spindle_assembly
slide_yz = Color([0.6,0.8,0.3,0], [Box(-250, 200, -250, 250, 3000, 250)])
spindle_yz = Collection([
             spindle_assembly,
             slide_yz
             ])
# move the spindle and it's vertical housing by the values set for the pivot lengths
# so the vismach origin is in the center of the spindle nose
spindle_yz = HalTranslate([spindle_yz],c,"pivot_y",0,1,0)
spindle_yz = HalTranslate([spindle_yz],c,"pivot_z",0,0,1)
spindle_yz = HalTranslate([spindle_yz],c,"offset_x",1,0,0)
spindle_yz = HalTranslate([spindle_yz],c,"offset_z",0,0,1)
# spindle head and y-slide move with y
spindle_yz = HalTranslate([spindle_yz],c,"axis_y",0,1,0)
# spindle head and y-slide move with z
spindle_yz = HalTranslate([spindle_yz],c,"axis_z",0,0,1)
# move the spindle_yz to it's designated home position
spindle_yz = Translate([spindle_yz], machine_zero_x, machine_zero_y, machine_zero_z)
#/tool-side

#work-side
work = Capture()
# create a simple cube
work_piece = BoxCenteredXY(600, 600, 600)

# create table
table = Color([0.1,0.7,0.9,0],[CylinderZ(0,920,-800,920)])
# create 'slots' in x and y on the table
slot_x_w = Color([1,1,1,1],[BoxCenteredXY(1840, 10, 0.9)])
slot_y_w = Color([1,1,1,1],[BoxCenteredXY(10, 1840, 0.9)])
# create indicator of at positive end of x slot
slot_pocket = Color([1,1,1,1],[CylinderZ(0,20,1,20)])
slot_pocket = Translate([slot_pocket],900,0,0)
table = Collection([
                 work,
                 work_piece,
                 table,
                 slot_x_w,
                 slot_y_w,
                 slot_pocket,
                 ])



table = Collection([
        table,
        coords,
        # block that carries the table 
        Color([0.5,0.5,0.5,0], [Box(-1500, -1000, -1300, 1500, 1000, -800)])
        ])
# table moves with x
table = HalTranslate([table],c,"axis_x",-1,0,0)
#/work-side

#base
base = Collection([
       # base
       Box(-3500, -1000, -1500, 3500, 3000, -1300),
       # column back
        Box(-2000, 1800, -1500, 2000, 4000, 3500)
       ])
base = Color([0.2,0.2,0.2,1], [base] )
#/base

model = Collection([table, spindle_yz, base,])

main(model, tooltip, work, size=4000, lat=-60, lon=25)
